@@ -8,7 +8,8 @@ import (
 	"github.com/docker/cli/cli/command"
 	"github.com/docker/cli/cli/debug"
 	"github.com/sirupsen/logrus"
-	"github.com/stretchr/testify/assert"
+	"gotest.tools/assert"
+	is "gotest.tools/assert/cmp"
 )
 
 func TestClientDebugEnabled(t *testing.T) {
@@ -18,15 +19,15 @@ func TestClientDebugEnabled(t *testing.T) {
 	cmd.Flags().Set("debug", "true")
 
 	err := cmd.PersistentPreRunE(cmd, []string{})
-	assert.NoError(t, err)
-	assert.Equal(t, "1", os.Getenv("DEBUG"))
-	assert.Equal(t, logrus.DebugLevel, logrus.GetLevel())
+	assert.NilError(t, err)
+	assert.Check(t, is.Equal("1", os.Getenv("DEBUG")))
+	assert.Check(t, is.Equal(logrus.DebugLevel, logrus.GetLevel()))
 }
 
 func TestExitStatusForInvalidSubcommandWithHelpFlag(t *testing.T) {
 	discard := ioutil.Discard
-	cmd := newDockerCommand(command.NewDockerCli(os.Stdin, discard, discard))
+	cmd := newDockerCommand(command.NewDockerCli(os.Stdin, discard, discard, false))
 	cmd.SetArgs([]string{"help", "invalid"})
 	err := cmd.Execute()
-	assert.EqualError(t, err, "unknown help topic: invalid")
+	assert.Error(t, err, "unknown help topic: invalid")
 }
